﻿Shader "NPR/PerPixelContoursReplacementShader"
{
	Properties
	{
		contourThreshold("Contour Threshold", Range(0.0,1.0)) = 0.3
		contourFalloff("Contour Falloff", Range(0.0,1.0)) = 0.05
	}
	SubShader
	{
		Tags{ "RenderType" = "Opaque" "LightMode" = "ForwardBase" }
		LOD 100

		Pass
		{
			Cull Off
			ZWrite Off
			ZTest LEqual
			Blend DstColor Zero

			CGPROGRAM
			#pragma vertex vert_contours
			#pragma fragment frag_contours

			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float3 normal : NORMAL;
			};

			struct v2f_toon
			{
				float3 normal : NORMAL;
				float3 viewDir : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f_toon vert_contours(appdata v)
			{
				v2f_toon o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.normal = UnityObjectToWorldNormal(v.normal);
				o.viewDir = WorldSpaceViewDir(v.vertex);
				return o;
			}

			float contourThreshold;
			float contourFalloff;
			float4 frag_contours(v2f_toon i) : SV_Target
			{
				float3 N = normalize(i.normal);
				float4 col = float4(1,1,1,1);
				float3 V = normalize(i.viewDir);
				float edgeValue = smoothstep(contourThreshold, contourThreshold + contourFalloff, dot(N,V));
				col.rgb *= edgeValue;
				return col;
			}
			ENDCG
		}
	}
}
