﻿Shader "NPR/BilateralFilter"
{
	Properties
	{
		_MainTex("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}

			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			float sigmaDistance;
			float sigmaValue;
			int filterDir;

			float Gauss(float d, float sigma)
			{
				return 1.0 / (sigma*sqrt(2.0*3.14)) * exp((-d*d) / (2.0*sigma*sigma));
			}

			float4 frag(v2f i) : SV_Target
			{
				float2 dir = filterDir == 0 ? float2(1,0) : float2(0,1);
				float3 SUM = float3(0,0,0);
				float SUMW = 0;
				if (sigmaDistance * sigmaValue == 0)
					SUM = tex2D(_MainTex, i.uv).rgb;
				else
				{
					float3 col0 = tex2D(_MainTex, i.uv).rgb;
						for (int ix = -5; ix <= 5; ix++)
						{
							float2 uv = i.uv + _MainTex_TexelSize.xy * dir * float(ix);
							float3 col = tex2D(_MainTex, uv).rgb;
							float wd = Gauss(length((i.uv - uv) / _MainTex_TexelSize.xy), sigmaDistance);
							float wv = Gauss(length(col-col0), sigmaValue);
							float w = wd*wv;
							SUMW += w;
							SUM += col*w;
						}
					SUM /= SUMW;
				}
				return float4(SUM, 1);
			}
			ENDCG
		}
	}
}
